"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createSpec = exports.getRoute = exports.redirect = exports.keyFilter = exports.createObject = exports.zip = exports.locationEqualsPath = exports.locationsEqual = exports.normalizeLocation = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _history = require("history");

var normalizeLocation = function normalizeLocation(location) {
  return (0, _history.parsePath)((0, _history.createPath)(location));
};

exports.normalizeLocation = normalizeLocation;

var locationsEqual = function locationsEqual(locationA, locationB) {
  return locationA.pathname === locationB.pathname && locationA.search === locationB.search && locationA.hash === locationB.hash;
};

exports.locationsEqual = locationsEqual;

var locationEqualsPath = function locationEqualsPath(location, path) {
  return locationsEqual(location, (0, _history.parsePath)(path));
};

exports.locationEqualsPath = locationEqualsPath;

var zip = function zip(a, b) {
  var zipped = [];

  for (var index = 0; index < a.length && index < b.length; index++) {
    zipped.push([a[index], b[index]]);
  }

  return zipped;
};

exports.zip = zip;

var createObject = function createObject(object, _ref) {
  var _ref2 = (0, _slicedToArray2.default)(_ref, 2),
      key = _ref2[0],
      value = _ref2[1];

  object[key] = value;
  return object;
};

exports.createObject = createObject;

var keyFilter = function keyFilter(object, condition) {
  return Object.keys(object).filter(condition).map(function (key) {
    return [key, object[key]];
  }).reduce(createObject, {});
};

exports.keyFilter = keyFilter;

var redirect = function redirect(routes, route) {
  if (route !== undefined && route.redirect !== undefined) {
    var redirected = routes.find(function (other) {
      return other.key === route.redirect;
    });
    return redirect(routes, redirected);
  } else {
    return route;
  }
};

exports.redirect = redirect;

var getRoute = function getRoute(routes, key) {
  var route = routes.find(function (route) {
    return route.key === key;
  });
  return redirect(routes, route);
};

exports.getRoute = getRoute;

var createSpec = function createSpec(route) {
  var values = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var paramValues = Object.values(route.params).map(function (_ref3) {
    var key = _ref3.key,
        notSetValue = _ref3.notSetValue;
    var value = key in values ? values[key] : notSetValue;
    return [key, value];
  }).reduce(createObject, {});
  return {
    route: route,
    params: paramValues
  };
};

exports.createSpec = createSpec;